/*
 * Test code for optparse.c
 *
 * Started: 2006-08-06
 * Updated: 2006-08-19
 */

#include <stdio.h>
#include <stdlib.h>

#include "optparse.h"

void display_help(void);
void display_version(void);


int main(int argc, char *argv[])
{
    /* count of options parsed/found */
    int     result   = 0;
    /* vars used for the parser */
    int     boolean  = 0;
    double  number   = 0.0;
    char   *output   = NULL;
    int     count    = 0;
    /* pointer to the first non-option argument */
    char   *filename = NULL;

    /* 
     * NOTE: casting puts()' return value to void keeps splint happy
     */

    (void)puts("Testing optparse:");

    /* initialize optparse */
    (void)puts("Initializing optparse...");
    optparse_init();
    optparse_help(display_help);
    optparse_version(display_version);

    /* populate parser */
    printf("adding (&flag_version, OPT_BOOL, 'b', \"boolean\"): " );
    if (optparse_add(&boolean, OPT_BOOL, 'b', "boolean",
                     "Some boolean value") == OPT_TRUE) {
        (void)puts("ok.");
    } else {
        (void)puts ("failed!");
        optparse_exit();
    }
    printf("adding (&number, OPT_DOUBLE, 'n', \"number\"): " );
    if (optparse_add(&number, OPT_DOUBLE, 'n', "number",
                     "Some floating point type") == OPT_TRUE) {
        (void)puts("ok.");
    } else {
        (void)puts("failed!");
        optparse_exit();
    }
    printf("adding (&output, OPT_STRING, 'o', \"output\"): " );
    if (optparse_add(&output, OPT_STRING, 'o', "output",
                     "Whatever...") == OPT_TRUE) {
        (void)puts("ok.");
    } else {
        (void)puts("failed!");
        optparse_exit();
    }
    printf("adding (&count, OPT_INT, 'c', \"count\"): " );
    if (optparse_add(&count, OPT_INT,'c', "count",
                     "Integer value") == OPT_TRUE) {
        (void)puts("ok.");
    } else {
        (void)puts("failed!");
        optparse_exit ();
    }

    /* run parser */
    (void)puts("running parser:\n");
    result = optparse_exec(argc, argv, &filename);
    
    switch (result) {
    
    case OPT_ERROR:
        (void)puts("failure while parsing, see messages...");
        break;
    case OPT_HELP:
        (void)puts("help requested, exiting normally...");
        break;
    case OPT_VERSION:
        (void)puts("version info requested, exiting normally...");
        break;
    default:
        printf("parser reports %d options have been parsed\n", result);
        printf("value of 'boolean' = %d\n", boolean);
        printf("value of 'count = %d\n", count);
        printf("value of 'number = %g\n", number);
        printf("value of 'output' = %s\n", output);
        printf("value of 'filename' = %s\n", filename);
        break;
    }

    /* destroy parser */
    optparse_exit();

    return EXIT_SUCCESS;
}


void display_help(void)
{
    (void)puts("Optparse tests --help--");
}

void display_version(void)
{
    (void)puts("Optparse version: 0.1-alpha");
}

